@@ -3,10 +3,11 @@
 
 import copy
 from functools import wraps
+import warnings
 
 import numpy as np
 
-from . import markup
+from . import markup, QuantitiesDeprecationWarning
 from .dimensionality import Dimensionality, p_dict
 from .registry import unit_registry
 from .decorators import with_doc
@@ -15,6 +16,9 @@ PREFERRED = []  # List of preferred quantities for each symbol,
                 # e.g. PREFERRED = [pq.mV, pq.pA, pq.UnitQuantity('femtocoulomb', 1e-15*pq.C, 'fC')]
                 # Intended to be overwritten in down-stream packages
 
+_np_version = tuple(map(int, np.__version__.split(".dev")[0].split(".")))
+
+
 def validate_unit_quantity(value):
     try:
         assert isinstance(value, Quantity)
@@ -114,15 +118,19 @@ class Quantity(np.ndarray):
     # TODO: what is an appropriate value?
     __array_priority__ = 21
 
-    def __new__(cls, data, units='', dtype=None, copy=True):
+    def __new__(cls, data, units='', dtype=None, copy=None):
+        if copy is not None:
+            warnings.warn(("The 'copy' argument in Quantity is deprecated and will be removed in the future. "
+                           "The argument has no effect since quantities-0.16.0 (to aid numpy-2.0 support)."),
+                           QuantitiesDeprecationWarning, stacklevel=2)
         if isinstance(data, Quantity):
             if units:
                 data = data.rescale(units)
             if isinstance(data, unit_registry['UnitQuantity']):
                 return 1*data
-            return np.array(data, dtype=dtype, copy=copy, subok=True).view(cls)
+            return np.asanyarray(data, dtype=dtype).view(cls)
 
-        ret = np.array(data, dtype=dtype, copy=copy).view(cls)
+        ret = np.asarray(data, dtype=dtype).view(cls)
         ret._dimensionality.update(validate_dimensionality(units))
         return ret
 
@@ -140,6 +148,16 @@ class Quantity(np.ndarray):
 
     @property
     def magnitude(self):
+        """
+        Returns a view onto the numerical value of the quantity, stripping
+        away the associated units. For example:
+        ```
+        import quantities as pq
+        t = 2 * pq.millisecond
+        n = t.magnitude # n will be 2 (not 0.002)
+        ```
+        See also: dimensionless_magnitude.
+        """
         return self.view(type=np.ndarray)
 
     @property
@@ -210,8 +228,8 @@ class Quantity(np.ndarray):
             dtype = self.dtype
         if self.dimensionality == to_dims:
             return self.astype(dtype)
-        to_u = Quantity(1.0, to_dims)
-        from_u = Quantity(1.0, self.dimensionality)
+        to_u = Quantity(1.0, to_dims, dtype=dtype)
+        from_u = Quantity(1.0, self.dimensionality, dtype=dtype)
         try:
             cf = get_conversion_factor(from_u, to_u)
         except AssertionError:
@@ -219,6 +237,8 @@ class Quantity(np.ndarray):
                 'Unable to convert between units of "%s" and "%s"'
                 %(from_u._dimensionality, to_u._dimensionality)
             )
+        if np.dtype(dtype).kind in 'fc':
+            cf = np.array(cf, dtype=dtype)
         new_magnitude = cf*self.magnitude
         dtype = np.result_type(dtype, new_magnitude)
         return Quantity(new_magnitude, to_u, dtype=dtype)
@@ -243,6 +263,23 @@ class Quantity(np.ndarray):
         raise Exception("Preferred units for '%s' (or equivalent) not specified in "
                         "quantites.quantity.PREFERRED." % self.dimensionality)
 
+    @property
+    def dimensionless_magnitude(self):
+        """
+        Returns the numerical value of a dimensionless quantity in the form of
+        a numpy array. Any decimal prefixes are normalized away first.
+        For example:
+        ```
+        import quantities as pq
+        t = 2 * pq.ms
+        f = 3 * pq.MHz
+        n = (t*f).dimensionless_magnitude # n will be 6000 (not 6)
+        ```
+        If the quantity is not dimensionless, a conversion error is raised.
+        See also: magnitude.
+        """
+        return self.rescale(unit_registry['dimensionless']).magnitude
+
     @with_doc(np.ndarray.astype)
     def astype(self, dtype=None, **kwargs):
         '''Scalars are returned as scalar Quantity arrays.'''
@@ -272,7 +309,7 @@ class Quantity(np.ndarray):
         uf, objs, huh = context
         if uf.__name__.startswith('is'):
             return obj
-        #print self, obj, res, uf, objs
+
         try:
             res._dimensionality = p_dict[uf](*objs)
         except KeyError:
@@ -283,11 +320,21 @@ class Quantity(np.ndarray):
                 )
         return res
 
-    def __array_wrap__(self, obj, context=None):
-        if not isinstance(obj, Quantity):
-            # backwards compatibility with numpy-1.3
-            obj = self.__array_prepare__(obj, context)
-        return obj
+    def __array_wrap__(self, obj, context=None, return_scalar=False):
+
+        # For NumPy < 2.0 we do old behavior
+        if _np_version < (2, 0, 0):
+            if not isinstance(obj, Quantity):
+                return self.__array_prepare__(obj, context)
+            else:
+                return obj
+        # For NumPy > 2.0 we either do the prepare or the wrap
+        else:
+            if not isinstance(obj, Quantity):
+                return self.__array_prepare__(obj, context)
+            else:
+                return super().__array_wrap__(obj, context, return_scalar)
+
 
     @with_doc(np.ndarray.__add__)
     @scale_other_units
@@ -298,7 +345,6 @@ class Quantity(np.ndarray):
     @scale_other_units
     def __radd__(self, other):
         return np.add(other, self)
-        return super().__radd__(other)
 
     @with_doc(np.ndarray.__iadd__)
     @scale_other_units
@@ -314,7 +360,6 @@ class Quantity(np.ndarray):
     @scale_other_units
     def __rsub__(self, other):
         return np.subtract(other, self)
-        return super().__rsub__(other)
 
     @with_doc(np.ndarray.__isub__)
     @scale_other_units
@@ -334,22 +379,46 @@ class Quantity(np.ndarray):
     @with_doc(np.ndarray.__imul__)
     @protected_multiplication
     def __imul__(self, other):
-        return super().__imul__(other)
+        # the following is an inelegant fix for the removal of __array_prepare__ in NumPy 2.x
+        # the longer-term solution is probably to implement __array_ufunc__
+        # See:
+        # - https://numpy.org/devdocs/release/2.0.0-notes.html#array-prepare-is-removed
+        # - https://numpy.org/neps/nep-0013-ufunc-overrides.html
+        if _np_version < (2, 0, 0):
+            return super().__imul__(other)
+        else:
+            cself = self.copy()
+            if hasattr(other, "copy"):
+                cother = other.copy()
+            else:
+                cother = other
+            res = super().__imul__(other)
+            context = (np.multiply, (cself, cother, cself), 0)
+            return self.__array_prepare__(res, context=context)
 
     @with_doc(np.ndarray.__rmul__)
     def __rmul__(self, other):
         return np.multiply(other, self)
-        return super().__rmul__(other)
 
     @with_doc(np.ndarray.__itruediv__)
     @protected_multiplication
     def __itruediv__(self, other):
-        return super().__itruediv__(other)
+        # see comment above on __imul__
+        if _np_version < (2, 0, 0):
+            return super().__itruediv__(other)
+        else:
+            cself = self.copy()
+            if hasattr(other, "copy"):
+                cother = other.copy()
+            else:
+                cother = other
+            res = super().__itruediv__(other)
+            context = (np.true_divide, (cself, cother, cself), 0)
+            return self.__array_prepare__(res, context=context)
 
     @with_doc(np.ndarray.__rtruediv__)
     def __rtruediv__(self, other):
         return np.true_divide(other, self)
-        return super().__rtruediv__(other)
 
     @with_doc(np.ndarray.__pow__)
     @check_uniform
@@ -360,7 +429,18 @@ class Quantity(np.ndarray):
     @check_uniform
     @protected_power
     def __ipow__(self, other):
-        return super().__ipow__(other)
+        # see comment above on __imul__
+        if _np_version < (2, 0, 0):
+            return super().__ipow__(other)
+        else:
+            cself = self.copy()
+            if hasattr(other, "copy"):
+                cother = other.copy()
+            else:
+                cother = other
+            res = super().__ipow__(other)
+            context = (np.power, (cself, cother, cself), 0)
+            return self.__array_prepare__(res, context=context)
 
     def __round__(self, decimals=0):
         return np.around(self, decimals)
@@ -476,7 +556,7 @@ class Quantity(np.ndarray):
         ret = self.magnitude.sum(axis, dtype, None if out is None else out.magnitude)
         dim = self.dimensionality
         if out is None:
-            return Quantity(ret, dim, copy=False)
+            return Quantity(ret, dim)
         if not isinstance(out, Quantity):
             raise TypeError("out parameter must be a Quantity")
         out._dimensionality = dim
@@ -484,11 +564,9 @@ class Quantity(np.ndarray):
 
     @with_doc(np.nansum)
     def nansum(self, axis=None, dtype=None, out=None):
-        import numpy as np
         return Quantity(
             np.nansum(self.magnitude, axis, dtype, out),
-            self.dimensionality,
-            copy=False
+            self.dimensionality
         )
 
     @with_doc(np.ndarray.fill)
@@ -523,7 +601,7 @@ class Quantity(np.ndarray):
     @with_doc(np.ndarray.searchsorted)
     def searchsorted(self,values, side='left'):
         if not isinstance (values, Quantity):
-            values = Quantity(values, copy=False)
+            values = Quantity(values)
 
         if values._dimensionality != self._dimensionality:
             raise ValueError("values does not have the same units as self")
@@ -539,7 +617,7 @@ class Quantity(np.ndarray):
         ret = self.magnitude.max(axis, None if out is None else out.magnitude)
         dim = self.dimensionality
         if out is None:
-            return Quantity(ret, dim, copy=False)
+            return Quantity(ret, dim)
         if not isinstance(out, Quantity):
             raise TypeError("out parameter must be a Quantity")
         out._dimensionality = dim
@@ -553,8 +631,7 @@ class Quantity(np.ndarray):
     def nanmax(self, axis=None, out=None):
         return Quantity(
             np.nanmax(self.magnitude),
-            self.dimensionality,
-            copy=False
+            self.dimensionality
         )
 
     @with_doc(np.ndarray.min)
@@ -562,7 +639,7 @@ class Quantity(np.ndarray):
         ret = self.magnitude.min(axis, None if out is None else out.magnitude)
         dim = self.dimensionality
         if out is None:
-            return Quantity(ret, dim, copy=False)
+            return Quantity(ret, dim)
         if not isinstance(out, Quantity):
             raise TypeError("out parameter must be a Quantity")
         out._dimensionality = dim
@@ -572,8 +649,7 @@ class Quantity(np.ndarray):
     def nanmin(self, axis=None, out=None):
         return Quantity(
             np.nanmin(self.magnitude),
-            self.dimensionality,
-            copy=False
+            self.dimensionality
         )
 
     @with_doc(np.ndarray.argmin)
@@ -588,12 +664,12 @@ class Quantity(np.ndarray):
     def nanargmax(self,axis=None, out=None):
         return np.nanargmax(self.magnitude)
 
-    @with_doc(np.ndarray.ptp)
+    @with_doc(np.ptp)
     def ptp(self, axis=None, out=None):
-        ret = self.magnitude.ptp(axis, None if out is None else out.magnitude)
+        ret = np.ptp(self.magnitude, axis, None if out is None else out.magnitude)
         dim = self.dimensionality
         if out is None:
-            return Quantity(ret, dim, copy=False)
+            return Quantity(ret, dim)
         if not isinstance(out, Quantity):
             raise TypeError("out parameter must be a Quantity")
         out._dimensionality = dim
@@ -604,8 +680,8 @@ class Quantity(np.ndarray):
         if min is None and max is None:
             raise ValueError("at least one of min or max must be set")
         else:
-            if min is None: min = Quantity(-np.Inf, self._dimensionality)
-            if max is None: max = Quantity(np.Inf, self._dimensionality)
+            if min is None: min = Quantity(-np.inf, self._dimensionality)
+            if max is None: max = Quantity(np.inf, self._dimensionality)
 
         if self.dimensionality and not \
                 (isinstance(min, Quantity) and isinstance(max, Quantity)):
@@ -620,7 +696,7 @@ class Quantity(np.ndarray):
         )
         dim = self.dimensionality
         if out is None:
-            return Quantity(clipped, dim, copy=False)
+            return Quantity(clipped, dim)
         if not isinstance(out, Quantity):
             raise TypeError("out parameter must be a Quantity")
         out._dimensionality = dim
@@ -631,7 +707,7 @@ class Quantity(np.ndarray):
         ret = self.magnitude.round(decimals, None if out is None else out.magnitude)
         dim = self.dimensionality
         if out is None:
-            return Quantity(ret, dim, copy=False)
+            return Quantity(ret, dim)
         if not isinstance(out, Quantity):
             raise TypeError("out parameter must be a Quantity")
         out._dimensionality = dim
@@ -642,7 +718,7 @@ class Quantity(np.ndarray):
         ret = self.magnitude.trace(offset, axis1, axis2, dtype, None if out is None else out.magnitude)
         dim = self.dimensionality
         if out is None:
-            return Quantity(ret, dim, copy=False)
+            return Quantity(ret, dim)
         if not isinstance(out, Quantity):
             raise TypeError("out parameter must be a Quantity")
         out._dimensionality = dim
@@ -652,8 +728,7 @@ class Quantity(np.ndarray):
     def squeeze(self, axis=None):
         return Quantity(
             self.magnitude.squeeze(axis),
-            self.dimensionality,
-            copy=False
+            self.dimensionality
         )
 
     @with_doc(np.ndarray.mean)
@@ -661,7 +736,7 @@ class Quantity(np.ndarray):
         ret = self.magnitude.mean(axis, dtype, None if out is None else out.magnitude)
         dim = self.dimensionality
         if out is None:
-            return Quantity(ret, dim, copy=False)
+            return Quantity(ret, dim)
         if not isinstance(out, Quantity):
             raise TypeError("out parameter must be a Quantity")
         out._dimensionality = dim
@@ -669,18 +744,16 @@ class Quantity(np.ndarray):
 
     @with_doc(np.nanmean)
     def nanmean(self, axis=None, dtype=None, out=None):
-        import numpy as np
         return Quantity(
             np.nanmean(self.magnitude, axis, dtype, out),
-            self.dimensionality,
-            copy=False)
+            self.dimensionality)
 
     @with_doc(np.ndarray.var)
     def var(self, axis=None, dtype=None, out=None, ddof=0):
         ret = self.magnitude.var(axis, dtype, out, ddof)
         dim = self._dimensionality**2
         if out is None:
-            return Quantity(ret, dim, copy=False)
+            return Quantity(ret, dim)
         if not isinstance(out, Quantity):
             raise TypeError("out parameter must be a Quantity")
         out._dimensionality = dim
@@ -691,7 +764,7 @@ class Quantity(np.ndarray):
         ret = self.magnitude.std(axis, dtype, out, ddof)
         dim = self.dimensionality
         if out is None:
-            return Quantity(ret, dim, copy=False)
+            return Quantity(ret, dim)
         if not isinstance(out, Quantity):
             raise TypeError("out parameter must be a Quantity")
         out._dimensionality = dim
@@ -701,8 +774,7 @@ class Quantity(np.ndarray):
     def nanstd(self, axis=None, dtype=None, out=None, ddof=0):
         return Quantity(
             np.nanstd(self.magnitude, axis, dtype, out, ddof),
-            self._dimensionality,
-            copy=False
+            self._dimensionality
         )
 
     @with_doc(np.ndarray.prod)
@@ -715,7 +787,7 @@ class Quantity(np.ndarray):
         ret = self.magnitude.prod(axis, dtype, None if out is None else out.magnitude)
         dim = self._dimensionality**power
         if out is None:
-            return Quantity(ret, dim, copy=False)
+            return Quantity(ret, dim)
         if not isinstance(out, Quantity):
             raise TypeError("out parameter must be a Quantity")
         out._dimensionality = dim
@@ -726,7 +798,7 @@ class Quantity(np.ndarray):
         ret = self.magnitude.cumsum(axis, dtype, None if out is None else out.magnitude)
         dim = self.dimensionality
         if out is None:
-            return Quantity(ret, dim, copy=False)
+            return Quantity(ret, dim)
         if not isinstance(out, Quantity):
             raise TypeError("out parameter must be a Quantity")
         out._dimensionality = dim
@@ -743,7 +815,7 @@ class Quantity(np.ndarray):
         ret = self.magnitude.cumprod(axis, dtype, out)
         dim = self.dimensionality
         if out is None:
-            return Quantity(ret, dim, copy=False)
+            return Quantity(ret, dim)
         if isinstance(out, Quantity):
             out._dimensionality = dim
         return out
@@ -766,10 +838,11 @@ class Quantity(np.ndarray):
                 (self.__class__, np.ndarray, (0, ), 'b', ),
                 state)
 
-    def __deepcopy__(self, memo_dict):
-        # constructor copies by default
-        return Quantity(self.magnitude, self.dimensionality)
-
+    def __deepcopy__(self, memo):
+        new_obj = super().__deepcopy__(memo).view(self.__class__)
+        new_obj.__dict__.update(self.__dict__)
+        memo[id(self)] = new_obj
+        return new_obj
 
 def _reconstruct_quantity(subtype, baseclass, baseshape, basetype,):
     """Internal function that builds a new MaskedArray from the
